﻿//
// Copyright (c) 2015, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MindFusion.Drawing;
using System.Drawing.Drawing2D;

using MF_Pen = MindFusion.Drawing.Pen;
using MF_Brush = MindFusion.Drawing.Brush;
using MF_PathGradientBrush = MindFusion.Drawing.PathGradientBrush;
using MF_LinearGradientBrush = MindFusion.Drawing.LinearGradientBrush;

using Pen = System.Drawing.Pen;
using Brush = System.Drawing.Brush;
using PathGradientBrush = System.Drawing.Drawing2D.PathGradientBrush;
using LinearGradientBrush = System.Drawing.Drawing2D.LinearGradientBrush;


namespace MindFusion.Gauges.WinForms.Samples.CS.Functions
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            gauge.CacheVisuals = false;

            OvalScale mainScale = gauge.GetElementByName("MainScale") as OvalScale;

            mainScale.CustomFunction = delegate(float value, float argument) 
            {
                float k = gauge.Scales[0].MinValue;
                float l = gauge.Scales[0].MaxValue;

                float a = (l - k) / ((float)Math.Pow(l, 2) - (float)Math.Pow(k, 2));
                float b = k - a * (float)Math.Pow(k, 2);

                return a * value * value + b;
            };

            mainScale.ReversedCustomFunction = delegate(float value, float argument)
            {
                float k = gauge.Scales[0].MinValue;
                float l = gauge.Scales[0].MaxValue;

                float a = (l - k) / ((float)Math.Pow(l, 2) - (float)Math.Pow(k, 2));
                float b = k - a * (float)Math.Pow(k, 2);

                return (float)Math.Sqrt((value - b) / a);
            };

            //Text scale element
            text = new Text();
            text.FontFamily = "Tahoma";
            text.Content = String.Format("{0:0.0}", 0);
            text.FontSize = new Length(14f, LengthType.Relative);
            text.FontStyle = FontStyle.Bold;
            text.Alignment = StringAlignment.Center;
            text.LineAlignment = StringAlignment.Center;
            text.Margin = new Thickness(0.3f, 0.5f, 0.3f, 0.3f);

            mainScale.ScaleChildren.Add(text);
            
            Pointer pointer = gauge.GetElementByName("ScalePointer") as Pointer;
            pointer.ValueChanged += pointer_ValueChanged;
        }

        private Text text;

        void pointer_ValueChanged(object sender, ValueChangedEventArgs<float> e)
        {
            text.Content = String.Format("{0:0.0}",e.NewValue);
        }

        private void gauge_PrepaintBackground(object sender, PrepaintEventArgs e)
        {
            e.CancelDefaultPainting = true;

            float widthExtent = 0;
            if (gauge.Width > gauge.Height)
                widthExtent = (float)(gauge.Width - gauge.Height) / (gauge.Height * 2);

            string figurePath = "M0.5,0 C0.75,0 1,0.3 1,0.75 C1,0.8 1,0.81 1,0.8 " +
                                "L0.57,0.9 C0.57,0.95 0.535,1 0.5,1 C0.465,1 0.43,0.95 " +
                                "0.43,0.9 L0,0.8 C0,0.3 0.25,0 0.5,0 Z";

            //Background base rectangle
            RoundRectangle rect = new RoundRectangle();
            rect.Roundness = 10;
            rect.Margin = new Thickness(-widthExtent, 0.05f, -widthExtent + 0.01f, 0.05f);
            rect.Fill = e.CreateLinearGradient(
                 45f,
                 0.2f, Color.FromArgb(0xA1, 0xAC, 0xBE),
                 0.8f, Color.FromArgb(0x71, 0x7C, 0x9E)
             );
            rect.Stroke = new MF_Pen(Color.FromArgb(0x5C, 0x69, 0x7A), 0);
            e.PaintVisualElement(rect, e.Element.RenderSize);

            //Shape of the gauge
            PathFigure baseShape = new PathFigure(figurePath);
            baseShape.Margin = new Thickness(-widthExtent + 0.035f, 0.095f, -widthExtent + 0.035f, 0.095f);
            baseShape.Fill = e.CreateLinearGradient(45f,
                0.0f, Color.FromArgb(0xFF, 0x90, 0x90, 0x90),
                0.2f, Color.FromArgb(0xFF, 0x90, 0x90, 0x90),
                0.8f, Color.FromArgb(0xFF, 0x30, 0x30, 0x30),
                1.0f, Color.FromArgb(0xFF, 0x30, 0x30, 0x30));
            e.PaintVisualElement(baseShape, e.Element.RenderSize);

            baseShape = new PathFigure(figurePath);
            baseShape.Margin = new Thickness(-widthExtent + 0.045f, 0.105f, -widthExtent + 0.045f, 0.105f);
            baseShape.Fill = e.CreateLinearGradient(45f,
                0.0f, Color.FromArgb(0xFF, 0x30, 0x30, 0x30),
                0.2f, Color.FromArgb(0xFF, 0x30, 0x30, 0x30),
                0.8f, Color.FromArgb(0xFF, 0x90, 0x90, 0x90),
                1.0f, Color.FromArgb(0xFF, 0x90, 0x90, 0x90));
            e.PaintVisualElement(baseShape, e.Element.RenderSize);

            baseShape = new PathFigure(figurePath);
            baseShape.Margin = new Thickness(-widthExtent + 0.05f, 0.11f, -widthExtent + 0.05f, 0.11f);
            baseShape.Fill = e.CreatePathGradient(new PointF(0.4f, 0.4f),
                0.15f, Color.FromArgb(0xEF, 0xF8, 0xFF),
                0.7f, Color.FromArgb(0xB0, 0xC4, 0xDE)
            );
            e.PaintVisualElement(baseShape, e.Element.RenderSize);

            //Background layer
            baseShape.Fill = e.CreatePathGradient(
                new PointF(0.5f, 0.85f),
                0.7f, Color.FromArgb(0x32, 0xFF, 0xFF, 0xFF),
                1f, Color.Transparent
            );
            e.PaintVisualElement(baseShape, e.Element.RenderSize);

            OvalScale mainScale = gauge.GetElementByName("MainScale") as OvalScale;

            //Text scale element background
            SizeF rs = e.Element.RenderSize;
            RectangleF bounds = new RectangleF(0.4f * rs.Width, 0.5f * rs.Height, 0.2f * rs.Width, 0.15f * rs.Height);

            GraphicsPath textPath = new GraphicsPath();
            textPath.AddEllipse(bounds);
            PathGradientBrush textFill = new PathGradientBrush(textPath);
            ColorBlend cb = new ColorBlend();
            cb.Colors = new Color[]
            {
                Color.Transparent,
                Color.Transparent,
                Color.FromArgb(0x40, 0x40, 0x00, 0x40)
            };
            cb.Positions = new float[] { 0, 0.2f, 1 };
            textFill.InterpolationColors = cb;
            e.Graphics.FillEllipse(textFill, bounds);
            textFill.Dispose();
            textPath.Dispose();
        }

        private void gauge_PrepaintForeground(object sender, PrepaintEventArgs e)
        {
            e.CancelDefaultPainting = true;

            float widthExtent = 0;
            if (gauge.Width > gauge.Height)
                widthExtent = (float)(gauge.Width - gauge.Height) / (gauge.Height * 2);

            //Reflection 1
            PathFigure figure1 = new PathFigure(
                "M0.51,0 C0.25,0 0.03,0.31 0.03,0.63 " +
                "C0.03,0.69 0.09,0.71 0.12,0.71 " + 
                "C0.16,0.71 0.21,0.68 0.21,0.64 " + 
                "C0.21,0.59 0.24,0.47 0.32,0.41 " +
                "C0.44,0.31 0.64,0.57 0.64,0.05 " + 
                "C0.64,0.03 0.55,0 0.51,0 Z");
            figure1.Fill = e.CreateLinearGradient(90f,
                0.05f, Color.Transparent,
                0.3f, Color.FromArgb(0x60, 0xFF, 0xFF, 0xFF),
                0.7f, Color.FromArgb(0x60, 0xFF, 0xFF, 0xFF),
                0.95f, Color.Transparent);
            figure1.Margin = new Thickness(-widthExtent + 0.05f, 0.11f, -widthExtent + 0.05f, 0.11f);
            e.PaintVisualElement(figure1, e.Element.RenderSize);

            //Reflection 2
            PathFigure figure2 = new PathFigure(
                "M0.8,0.32 L0.71,0.34 C0.795,0.37 0.89,0.44 0.95,0.54 " + 
                "C0.95,0.47 0.9,0.36 0.8,0.32 Z");
            figure2.Fill = e.CreateLinearGradient(90f,
                0.05f, Color.Transparent,
                0.3f, Color.FromArgb(0x60, 0xFF, 0xFF, 0xFF),
                0.7f, Color.FromArgb(0x60, 0xFF, 0xFF, 0xFF),
                0.95f, Color.Transparent);
            figure2.Margin = new Thickness(-widthExtent + 0.05f, 0, -widthExtent + 0.05f, 0);
            e.PaintVisualElement(figure2, e.Element.RenderSize);

        }

        private void gauge_PrepaintPointer(object sender, PrepaintEventArgs e)
        {
            if (e.Element.Name == "ScalePointer")
            {
                e.CancelDefaultPainting = true;

                //Hand of the pointer
                RectangleF bounds = Utils.ToRectangleF(e.Element.RenderSize);

                Pen stroke = new Pen(Color.FromArgb(0x32, 0x32, 0x32), 0.005f); 

                bounds.Inflate(0, -0.4f * bounds.Height);

                LinearGradientBrush fillRect = new LinearGradientBrush(
                    bounds, Color.Transparent, Color.Transparent, 90, true);

                ColorBlend blend = new ColorBlend();
                blend.Colors = new Color[] {
                    Color.FromArgb(0x81, 0x81, 0x81),
                    Color.FromArgb(0x60, 0x60, 0x60),
                    Color.FromArgb(0x32, 0x32, 0x32),
                    Color.FromArgb(0x0, 0x0, 0x0)
                };
                blend.Positions = new float[] { 0.0f, 0.25f, 0.75f, 1.0f };

                fillRect.InterpolationColors = blend;

                e.Graphics.DrawRectangle(stroke, bounds.X, bounds.Y, 
                    bounds.Width, bounds.Height);
                e.Graphics.FillRectangle(fillRect, bounds);

                fillRect.Dispose();

                //Base of the pointer
                bounds = new RectangleF(0, 0, 0.125f * e.Element.RenderSize.Width, e.Element.RenderSize.Height);

                GraphicsPath p = new GraphicsPath();
                p.AddEllipse(bounds);

                PathGradientBrush fillEllipse = new PathGradientBrush(p);

                fillEllipse.CenterColor = Color.White;
                fillEllipse.SurroundColors = new Color[] { Color.Black };

                fillEllipse.FocusScales = new PointF(0.25f, 0.25f);

                e.Graphics.DrawEllipse(stroke, bounds);
                e.Graphics.FillEllipse(fillEllipse, bounds);

                fillEllipse.Dispose();
                p.Dispose();
             }
        }

        private void rbLinear_CheckedChanged(object sender, EventArgs e)
        {
            if (rbLinear.Checked)
                gauge.Scales[0].FunctionType = FunctionType.Linear;
        }

        private void rbLog_CheckedChanged(object sender, EventArgs e)
        {
            if (rbLog.Checked)
                gauge.Scales[0].FunctionType = FunctionType.Logarithmic;
        }

        private void rbCustom_CheckedChanged(object sender, EventArgs e)
        {
            if (rbCustom.Checked)
                gauge.Scales[0].FunctionType = FunctionType.Custom;
        }

        private void panel2_Resize(object sender, EventArgs e)
        {
            //Keep gauge's size and relative position in the center
            gauge.Bounds = new Rectangle(
                (sender as Panel).Width / 2 - 178,
                (sender as Panel).Height / 2 - 128,
                356, 256);
        }
    }
}